from copy import copy
import random
from pythonsnake.snake import Snake
from pythonsnake.candy import Candy


class Game:
    def __init__(self, grid_width: int, grid_height: int) -> None:
        self.grid_width = grid_width
        self.grid_height = grid_height
        self.snake = Snake()
        self.lost = False
        self.current_direction = "right"
        self.candy = Candy(grid_width, grid_height, self.snake)
        self.grid = self.init_grid(grid_width, grid_height)
        self.update_grid()

    def move(self, direction: str):
        # if the User is not trying to U turn, process the move
        if not self.is_U_turn(direction):

            # if the move is allowed (not off grid or snake not biting itself)
            if self.is_move_allowed(direction):
                self.current_direction = direction.replace(" arrow", "")
                self.snake.move(direction)
                if self.check_candy():
                    self.snake.eat_candy()
                    self.candy.new_coord(
                        self.grid_width, self.grid_height, self.snake
                    )

                self.update_grid()
            else:
                self.lost = True

        return self

    def exit_game(self):
        self.lost = True

    """
    check if the move is allowed in the grid
    """

    def is_move_allowed(self, direction: str):

        snake_head = copy(self.snake.coord[0])
        if direction == "left arrow":
            snake_head["x"] -= 1
        elif direction == "right arrow":
            snake_head["x"] += 1
        elif direction == "up arrow":
            snake_head["y"] -= 1
        elif direction == "down arrow":
            snake_head["y"] += 1

        # check if next position of snake head is off grid
        if (
            snake_head["x"] < 0
            or snake_head["x"] >= self.grid_width
            or snake_head["y"] < 0
            or snake_head["y"] >= self.grid_height
        ):
            return False

        # check if next position of snake head is not another snake piece
        for snake_piece in self.snake.coord:
            if (
                snake_piece["x"] == snake_head["x"]
                and snake_piece["y"] == snake_head["y"]
            ):
                return False

        return True

    def new_candy(self):
        return [
            random.randint(0, self.grid_size),
            random.randint(0, self.grid_size),
        ]

    def check_candy(self) -> bool:
        #if snake head and candy at the same position
        if (
            self.snake.coord[0]["x"] == self.candy.X
            and self.snake.coord[0]["y"] == self.candy.Y
        ):
            return True

        return False

    def is_U_turn(self, direction: str) -> bool:
        # the User/snake can't make a U turn
        if (
            direction == "left arrow"
            and self.current_direction == "right"
            or direction == "right arrow"
            and self.current_direction == "left"
            or direction == "up arrow"
            and self.current_direction == "down"
            or direction == "down arrow"
            and self.current_direction == "up"
        ):
            return True

        return False

        """
        _summary_

        init the playing grid with format
                            column = width
        height = Y1 = line = [ [X1],[X2],[X3],[X4] ]
                        Y2     [X1],[X2],[X3],[X4]
                        Y3     [X1],[X2],[X3],[X4]
                        Y4     [X1],[X2],[X3],[X4]
        """

    def init_grid(self, grid_width: int, grid_height: int) -> None:
        return [
            [{"snake": False, "candy": False} for i in range(0, grid_width)]
            for j in range(0, grid_height)
        ]

    def update_grid(self) -> None:
        # reset grid
        for line in self.grid:
            for column in line:
                column["snake"] = False
                column["candy"] = False

        for snake_piece in self.snake.coord:
            self.grid[snake_piece["y"]][snake_piece["x"]]["snake"] = True

        self.grid[self.candy.Y][self.candy.X]["candy"] = True
